# VCS routines for musdex
#
# Copyright 2010 Max Battcher. Some rights reserved.
# Licensed for use under the Ms-RL. See attached LICENSE file.
from config import BASEDIR, load_index, save_index, save_config
from formatters import get_formatter
from handlers import get_handler
import datetime
import logging
import os
import os.path
import re
import shutil
import time

import vcs

def _mtime(F):
    return datetime.datetime(*time.localtime(os.path.getmtime(F))[:6])

def add(args, config):
    index = load_index(config)

    for archive in args.archive:
        archive = os.path.relpath(archive)
        if 'archives' in config \
        and any(arc['filename'] == archive for arc in config['archives']):
            logging.warn("Archive already configured: %s" % archive)
            continue

        handler = get_handler(args.handler)
        arch = handler(archive, os.path.join(BASEDIR, archive))
        if not arch.check():
            logging.error("Archive not supported by given handler: %s: %s" % (
                args.handler, archive))

        logging.info("Extracting archive for the first time: %s" % archive)
        files = arch.extract(force=True)
        for f, t in files:
            index[f] = t
            vcs.add_file(config, f)

        entry = {'filename': archive}
        if args.handler: entry['handler'] = args.handler
        if 'archives' not in config: config['archives'] = []
        config['archives'].append(entry)

    save_config(args, config)
    save_index(config, index)

def remove(args, config):
    index = load_index(config)

    if 'archives' not in config:
        logging.error("No archives have been configured.")
        return

    manifest = vcs.manifest(config)

    for archive in args.archive:
        archive = os.path.relpath(archive)
        arcloc = os.path.join(BASEDIR, archive)

        if not any(arc['filename'] == archive for arc in config['archives']):
            logging.warn("Archive not configured: %s" % archive)
            continue

        logging.info("Removing archive files from VCS.")
        for f in manifest:
            if f.startswith(arcloc):
                vcs.remove_file(config, f)

                if f in index:
                    del index[f]

        config['archives'] = [arc for arc in config['archives'] \
            if arc['filename'] != archive]
    
    save_config(args, config)
    save_index(config, index)

def extract(args, config):
    index = load_index(config)
    index_updated = False

    fmts = []
    if 'post_extract' in config:
        logging.debug("Compiling post-extraction regular expressions")
        fmts = [(re.compile(regex), get_formatter(fname)) \
            for regex, fname in config['post_extract']]

    if args.archive: args.archive = map(os.path.relpath, args.archive)

    manifest = vcs.manifest(config)

    for archive in config['archives']:
        arcf = archive['filename']
        arcloc = os.path.join(BASEDIR, arcf)
        if args.archive and arcf not in args.archive:
            continue

        # Check if up to date
        if not args.force and arcloc in index \
        and _mtime(arcf) <= index[arcloc]:
            continue

        arcman = dict((f, index[f] if f in index else None) \
            for f in manifest if f.startswith(arcloc))

        hname = archive['handler'] if 'handler' in archive else None
        handler = get_handler(hname)
        arch = handler(arcf, arcloc, manifest=arcman)
        files = arch.extract(force=args.force or arcloc not in index)
        if files: index_updated = True

        for f, t in files:
            if t is None: # File was removed
                del index[f]
                vcs.remove_file(config, f)
                continue
            index[f] = t
            for regex, fmt in fmts: # post-extract formatters
                if regex.match(f):
                    logging.debug("Post-extraction: %s(%s)" % (fmt, f))
                    fmt(f)
            if f != arcloc and f not in arcman: vcs.add_file(config, f)
            
    if index_updated: save_index(config, index)

def combine(args, config):
    index = load_index(config)
    index_updated = False

    if args.archive: args.archive = map(os.path.relpath, args.archive)

    manifest = vcs.manifest(config)

    for archive in config['archives']:
        arcf = archive['filename']
        arcloc = os.path.join(BASEDIR, arcf)
        if args.archive and arcf not in args.archive:
            continue

        arcman = dict((f, index[f] if f in index else None) \
            for f in manifest if f.startswith(arcloc))

        logging.debug("Checking modification times for %s" % arcf)
        # Unless forced or first-time combination, we do a quick sanity
        # check to see if any of the archive's files have changed
        if not args.force and arcloc in index and not \
        any(_mtime(f) > arcman[f] if arcman[f] is not None else True \
        for f in arcman):
            continue

        bakfilename = None
        if 'backup' not in config or config['backup']:
            logging.debug('Backing up %s' % arcf)
            bakfilename = arcf + '.bak~'
            shutil.copyfile(arcf, bakfilename)

        hname = archive['handler'] if 'handler' in archive else None
        handler = get_handler(hname)
        arch = handler(arcf, arcloc, manifest=arcman)
        files = arch.combine(force=args.force or arcloc not in index)
        if files: index_updated = True

        for f, t in files:
            index[f] = t

        if bakfilename is not None and ('leave_backups' not in config
        or not config['leave_backups']):
            logging.debug('Removing backup %s' % bakfilename)
            os.remove(bakfilename)

    if index_updated: save_index(config, index)

# vim: ai et ts=4 sts=4 sw=4



#!/usr/bin/python
# musdex - The Version Control-Aware Zip Archive Tool
# Named after a spell briefly mentioned in Sorcerer magazine
# But can also stand for "Multi-Unit Single-Document EXtractor"
#
# Copyright 2010 Max Battcher. Some rights reserved.
# Licensed for use under the Ms-RL. See attached LICENSE file.

def main(booznik=False):
    from config import load_config
    import argparse
    import commands
    import logging
    import sys

    parser = argparse.ArgumentParser(prog='musdex' if not booznik else 'xedsum')
    parser.add_argument('--config', '-c')
    parser.add_argument('--verbose', '-v', action="store_true", default=False)
    parser.add_argument('--quiet', '-q', action="store_true", default=False)
    subparsers = parser.add_subparsers()

    parser_extract = subparsers.add_parser('extract')
    parser_extract.add_argument('--force', '-f', action="store_true",
        default=False)
    parser_extract.add_argument('archive', nargs='*')
    parser_extract.set_defaults(func=commands.extract)
    
    parser_combine = subparsers.add_parser('combine')
    parser_combine.add_argument('--force', '-f', action="store_true",
        default=False)
    parser_combine.add_argument('archive', nargs='*')
    parser_combine.set_defaults(func=commands.combine)

    parser_add = subparsers.add_parser('add')
    parser_add.add_argument('--handler')
    parser_add.add_argument('archive', nargs='+')
    parser_add.set_defaults(func=commands.add)

    parser_remove = subparsers.add_parser('remove')
    parser_remove.add_argument('archive', nargs='+')
    parser_remove.set_defaults(func=commands.remove)

    args = sys.argv[1:]
    if len(args) == 0:
        args = ['extract'] if not booznik else ['combine']
    args = parser.parse_args(args)
    if args.verbose:
        logging.basicConfig(level=logging.DEBUG)
    elif not args.quiet:
        logging.basicConfig(level=logging.INFO)
    config = load_config(args)
    args.func(args, config)

if __name__ == "__main__":
    main()

# vim: ai et ts=4 sts=4 sw=4



# Configuration system for musdex
#
# Copyright 2010 Max Battcher. Some rights reserved.
# Licensed for use under the Ms-RL. See attached LICENSE file.
import logging
import os
import os.path
import yaml

import vcs

BASEDIR = "_musdex"
DEFAULT_CONFIG = os.path.join(BASEDIR, "musdex.yaml")
DEFAULT_INDEX = os.path.join(BASEDIR, ".musdex.index.yaml")

def load_config(args):
    conf = DEFAULT_CONFIG
    if args.config:
        conf = args.config
    logging.debug("Loading configuration from %s" % conf)
    if not os.path.exists(conf):
        logging.info("No configuration file found at %s" % conf)
        return {}
    f = open(conf, 'r')
    config = yaml.load(f)
    f.close()
    return config

def save_config(args, config):
    conf = DEFAULT_CONFIG
    if args.config:
        conf = args.config
    logging.debug("Saving configuration to %s" % conf)
    confdir = os.path.dirname(conf)
    if confdir and not os.path.exists(confdir):
        logging.info("Config directory does not exist: %s" % confdir)
        os.makedirs(confdir)
    new_conf = not os.path.exists(conf)
    f = open(conf, 'w')
    yaml.dump(config, f)
    f.close()
    if new_conf:
        logging.info("Adding new configuration file to vcs: %s" % conf)
        vcs.add_file(config, conf)

# TODO: Perhaps the index should instead be a less verbose format?
def load_index(config):
    index = config['index'] if 'index' in config else DEFAULT_INDEX
    if os.path.exists(index):
        logging.debug("Loading existing index: %s" % index)
        f = open(index, 'r')
        index = yaml.load(f)
        f.close()
        return index
    return {}

def save_index(config, index):
    idx = config['index'] if 'index' in config else DEFAULT_INDEX
    logging.debug("Saving index: %s" % idx)
    idxdir = os.path.dirname(idx)
    if idxdir and not os.path.exists(idxdir):
        logging.info("Index directory does not exist: %s" % idxdir)
        os.makedirs(idxdir)
    f = open(idx, 'w')
    yaml.dump(index, f)
    f.close()

# vim: ai et ts=4 sts=4 sw=4
